@@ -17,6 +17,7 @@
 #include <string.h>
 
 #include "asm.h"
+#include "strcmpi.h"
 
 /*)Module	asmain.c
  *
@@ -479,7 +480,7 @@ asmbl()
 	struct expr e1;
 	char id[NCPS];
 	char opt[NCPS];
-	char fn[FILENAME_MAX];
+	char fn[PATH_MAX];
 	char *p;
 	int d, n, uaf, uf;
 
@@ -799,7 +800,7 @@ loop:
 		break;
 
 	case S_MODUL:
-		getst(id, -1);
+	        getst(id, getnb()); // a module can start with a digit
 		if (pass == 0) {
 			if (module[0]) {
 				err('m');
@@ -810,6 +811,20 @@ loop:
 		lmode = SLIST;
 		break;
 
+    case S_OPTSDCC:
+		p = optsdcc;
+		if ((c = getnb()) != 0) {
+			do {
+				if (p < &optsdcc[NINPUT-1])
+					*p++ = c;
+			} while ((c = get()) != 0);
+		}
+		*p = 0;
+		unget(c);
+		lmode = SLIST;
+        /*if (pass == 0) printf("optsdcc=%s\n", optsdcc);*/
+        break;
+
 	case S_GLOBL:
 		do {
 			getid(id, -1);
@@ -843,10 +858,6 @@ loop:
 		if ((ap = alookup(id)) != NULL) {
 			if (uaf && uf != ap->a_flag)
 				err('m');
-                        if (ap->a_flag & A_OVR) {
-				ap->a_size = 0; 
-                                ap->a_fuzz=0;
-			}
 		} else {
 			ap = (struct area *) new (sizeof(struct area));
 			ap->a_ap = areap;
@@ -911,7 +922,7 @@ loop:
 		d = getnb();
 		p = fn;
 		while ((c = get()) != d) {
-			if (p < &fn[FILENAME_MAX-1]) {
+			if (p < &fn[PATH_MAX-1]) {
 				*p++ = c;
 			} else {
 				break;
@@ -935,7 +946,7 @@ loop:
 		{
 		    getst(id, -1);
 		    
-		    if (!strcmpi(id, "on"))
+		    if (!as_strcmpi(id, "on"))
 		    {
 		    	/* Quick sanity check: size of 
 		    	 * Addr_T must be at least 24 bits.
@@ -953,7 +964,7 @@ loop:
 		    	    flat24Mode = 1;
 		    	}
 		    }
-		    else if (!strcmpi(id, "off"))
+		    else if (!as_strcmpi(id, "off"))
 		    {
 		        flat24Mode = 0;
 		    }
@@ -1053,8 +1064,8 @@ int wf;
 	p2 = strrchr (afn, FSEPX);		// search last '.'
 	if (!p2)
 		p2 = afn + strlen (afn);
-	if (p2 > &afn[FILENAME_MAX-4])		// truncate filename, if it's too long
-		p2 = &afn[FILENAME_MAX-4];
+	if (p2 > &afn[PATH_MAX-4])		// truncate filename, if it's too long
+		p2 = &afn[PATH_MAX-4];
 	*p2++ = FSEPX;
 
 	// choose a file-extension
@@ -1067,7 +1078,7 @@ int wf;
 	}
 
 	while ((c = *p3++) != 0) {		// strncpy
-		if (p2 < &afn[FILENAME_MAX-1])
+		if (p2 < &afn[PATH_MAX-1])
 			*p2++ = c;
 	}
 	*p2++ = 0;
@@ -1114,11 +1125,26 @@ register struct area *nap;
 	register struct area *oap;
 
 	oap = dot.s_area;
+	/* fprintf (stderr, "%s dot.s_area->a_size: %d dot.s_addr: %d\n",
+	     oap->a_id, dot.s_area->a_size, dot.s_addr); */
 	oap->a_fuzz = fuzz;
-	oap->a_size = dot.s_addr;
-	fuzz = nap->a_fuzz;
+	if (oap->a_flag & A_OVR) {
+	  // the size of an overlay is the biggest size encountered
+	  if (oap->a_size < dot.s_addr) {
+	    oap->a_size = dot.s_addr;
+	  }
+	} else {
+	  oap->a_size = dot.s_addr;
+	}
+	if (nap->a_flag & A_OVR) {
+	  // a new overlay starts at 0, no fuzz
+	  dot.s_addr = 0;
+	  fuzz = 0;
+	} else {
+	  dot.s_addr = nap->a_size;
+	  fuzz = nap->a_fuzz;
+	}
 	dot.s_area = nap;
-	dot.s_addr = nap->a_size;
 	outall();
 }
 
