@@ -1,4 +1,4 @@
-#include <limits.h>
+#include <float.h>
 
 /* convert float to signed char */
 signed char __fs2schar (float f) {
