@@ -1,4 +1,4 @@
-#include <limits.h>
+#include <float.h>
 
 /* convert float to signed int */
 signed int __fs2sint (float f) {
