@@ -1,5 +1,6 @@
+#include <float.h>
+
 /* convert unsigned char to float */
 float __uchar2fs (unsigned char uc) {
-  unsigned long ul=uc;
-  return __ulong2fs(ul);
+  return __ulong2fs(uc);
 }
