@@ -1,5 +1,6 @@
+#include <float.h>
+
 /* convert unsigned int to float */
 float __uint2fs (unsigned int ui) {
-  unsigned long ul=ui;
-  return __ulong2fs(ul);
+  return __ulong2fs(ui);
 }
