@@ -30,10 +30,12 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 #include "ddconfig.h"
 
+// prj
 #include "stypes.h"
+#include "appcl.h"
 
 
-//extern class cl_sim *simulator;
+extern class cl_app *application;
 
 extern char delimiters[];
 
