@@ -1,8 +1,8 @@
 /*-------------------------------------------------------------------------
 
-  SDCCglue.h - glues everything we have done together into one file.                 
+  SDCCglue.h - glues everything we have done together into one file.
                 Written By -  Sandeep Dutta . sandeep.dutta@usa.net (1998)
-		
+
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2, or (at your option) any
@@ -21,15 +21,10 @@
    You are forbidden to forbid anyone else to use, share and improve
    what you give them.   Help stamp out software-hoarding!  
 -------------------------------------------------------------------------*/
-#include "SDCCglobl.h"
-#include "SDCCmem.h"
 
-#ifndef SDCCGLUE_H
-#define SDCCGLUE_H 1
+#ifndef PIC_GLUE_H
+#define PIC_GLUE_H
 
-void glue ();
-/* drdani Jan 30 2000
-   This is needed in gen.c of z80 port */
-char *aopLiteral (value *, int);
+void picglue (void);
 
 #endif
