@@ -166,6 +166,8 @@ struct spec_nodes
   cpp_hashnode *n__STRICT_ANSI__;	/* STDC_0_IN_SYSTEM_HEADERS */
   cpp_hashnode *n__CHAR_UNSIGNED__;	/* plain char is unsigned */
   cpp_hashnode *n__VA_ARGS__;		/* C99 vararg macros */
+  /* SDCC _asm specific */
+  cpp_hashnode *n__asm; 		/* _asm ... _endasm ; */
 };
 
 struct cpp_buffer
